@@ -44,9 +44,12 @@
 #endif
 
 /* Must be before all other gnome includes!! */
-#include "gnome-i18nP.h"
+#include <glib/gi18n-lib.h>
 
+#include <gdkconfig.h>
+#ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
+#endif
 #include <libgnome/libgnome.h>
 #include <bonobo/bonobo-ui-main.h>
 #include <gconf/gconf-client.h>
@@ -73,6 +76,7 @@
 #include <gtk/gtkcheckmenuitem.h>
 #include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkaboutdialog.h>
+#include <gtk/gtklinkbutton.h>
 
 /*****************************************************************************
  * libgnomeui
@@ -112,29 +116,9 @@ static gboolean use_event_sounds;
 /* GConf client for monitoring the event sounds option */
 static GConfClient *gconf_client = NULL;
 
-
 enum { ARG_DISABLE_CRASH_DIALOG=1, ARG_DISPLAY };
 
-static struct poptOption libgnomeui_options[] = {
-        {NULL, '\0', POPT_ARG_INTL_DOMAIN, GETTEXT_PACKAGE, 0, NULL, NULL},
-	{NULL, '\0', POPT_ARG_CALLBACK|POPT_CBFLAG_PRE|POPT_CBFLAG_POST,
-	 &libgnomeui_arg_callback, 0, NULL, NULL},
-	{"disable-crash-dialog", '\0', POPT_ARG_NONE, NULL, ARG_DISABLE_CRASH_DIALOG,
-        N_("Disable Crash Dialog"), NULL},
-        {"display", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, NULL, ARG_DISPLAY,
-         N_("X display to use"), N_("DISPLAY")},
-	{NULL, '\0', 0, NULL, 0, NULL, NULL}
-};
-
-static GOptionEntry libgnomeui_goptions[] = {
-	{ "disable-crash-dialog", '\0', G_OPTION_FLAG_NO_ARG,
-	  G_OPTION_ARG_CALLBACK, libgnomeui_goption_disable_crash_dialog,
-	  N_("Disable Crash Dialog"), NULL },
-	/* --display is already handled by gtk+ */
-	{ NULL }
-};
-
-void _gnome_ui_gettext_init (gboolean) G_GNUC_INTERNAL;
+G_GNUC_INTERNAL void _gnome_ui_gettext_init (gboolean);
 
 void G_GNUC_INTERNAL
 _gnome_ui_gettext_init (gboolean bind_codeset)
@@ -161,12 +145,19 @@ _gnome_ui_gettext_init (gboolean bind_codeset)
 static GOptionGroup *
 libgnomeui_get_goption_group (void)
 {
+	const GOptionEntry libgnomeui_goptions[] = {
+		{ "disable-crash-dialog", '\0', G_OPTION_FLAG_NO_ARG,
+		  G_OPTION_ARG_CALLBACK, libgnomeui_goption_disable_crash_dialog,
+		  N_("Disable Crash Dialog"), NULL },
+		  /* --display is already handled by gtk+ */
+		{ NULL }
+	};
 	GOptionGroup *option_group;
 
 	_gnome_ui_gettext_init (TRUE);
 
 	option_group = g_option_group_new ("gnome-ui",
-					   N_("GNOME GUI Library"),
+					   N_("GNOME GUI Library:"),
 					   N_("Show GNOME GUI options"),
 					   NULL, NULL);
 	g_option_group_set_translation_domain (option_group, GETTEXT_PACKAGE);
@@ -178,6 +169,17 @@ libgnomeui_get_goption_group (void)
 const GnomeModuleInfo *
 libgnomeui_module_info_get (void)
 {
+	static struct poptOption libgnomeui_options[] = {
+		{ NULL, '\0', POPT_ARG_INTL_DOMAIN, GETTEXT_PACKAGE, 0, NULL, NULL },
+		{ NULL, '\0', POPT_ARG_CALLBACK|POPT_CBFLAG_PRE|POPT_CBFLAG_POST,
+		  &libgnomeui_arg_callback, 0, NULL, NULL },
+		{ "disable-crash-dialog", '\0', POPT_ARG_NONE, NULL, ARG_DISABLE_CRASH_DIALOG,
+		  N_("Disable Crash Dialog"), NULL },
+		{ "display", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, NULL, ARG_DISPLAY,
+		  N_("X display to use"), N_("DISPLAY") },
+		{ NULL, '\0', 0, NULL, 0, NULL, NULL }
+	};
+
 	static GnomeModuleInfo module_info = {
 		"libgnomeui", VERSION, N_("GNOME GUI Library"),
 		NULL, libgnomeui_instance_init,
@@ -326,7 +328,7 @@ libgnomeui_set_property (GObject *object, guint param_id,
         priv = g_object_get_qdata(G_OBJECT(program), quark_gnome_program_private_libgnomeui);
 
 	if (param_id == cdata->default_icon_id)
-		priv->default_icon = g_strdup (g_value_get_string (value));
+		priv->default_icon = g_value_dup_string (value);
 	else if (param_id == cdata->crash_dialog_id)
 		priv->show_crash_dialog = g_value_get_boolean (value) != FALSE;
 	else if (param_id == cdata->display_id)
@@ -793,13 +795,9 @@ libgnomeui_segv_setup (GnomeProgram *program)
         static struct sigaction *setptr;
         struct sigaction sa;
         gboolean do_crash_dialog = TRUE;
-        GValue value = { 0, };
 
-        g_value_init (&value, G_TYPE_BOOLEAN);
-        g_object_get_property (G_OBJECT (program),
-                               LIBGNOMEUI_PARAM_CRASH_DIALOG, &value);
-        do_crash_dialog = g_value_get_boolean (&value);
-        g_value_unset (&value);
+        g_object_get (program, LIBGNOMEUI_PARAM_CRASH_DIALOG,
+		      &do_crash_dialog, NULL);
 
         if(do_crash_dialog) {
                 memset(&sa, 0, sizeof(sa));
@@ -877,20 +875,24 @@ static void libgnomeui_segv_handle(int signum)
 		_exit(1);
 	} else /* pid == 0 */ {
                 GnomeProgram *program;
+                const char *app_version;
 		char buf[32];
 
 		g_snprintf(buf, sizeof(buf), "%d", signum);
 
                 program = gnome_program_get();
+                if (program) {
+                  app_version = gnome_program_get_app_version (program);
+                } else {
+                  app_version = "(indeterminate)";
+                }
 
 		/* Child process */
 		execl (LIBGNOMEUI_SERVERDIR "/gnome_segv2", LIBGNOMEUI_SERVERDIR "/gnome_segv",
-		       g_get_prgname (), buf,
-		       gnome_program_get_app_version (program), NULL);
+		       g_get_prgname (), buf, app_version, NULL);
 
                 execlp ("gnome_segv2", "gnome_segv2",
-			g_get_prgname (), buf,
-			gnome_program_get_app_version (program), NULL);
+			g_get_prgname (), buf, app_version, NULL);
 
                 _exit(99);
 	}
@@ -942,13 +944,9 @@ gnome_init_with_popt_table (const char *app_id,
 				      NULL);
 
         if(return_ctx) {
-                GValue value = { 0, };
-
-                g_value_init (&value, G_TYPE_POINTER);
-                g_object_get_property (G_OBJECT (program),
-                                       GNOME_PARAM_POPT_CONTEXT, &value);
-                *return_ctx = g_value_peek_pointer (&value);
-                g_value_unset (&value);
+                poptContext context;
+                g_object_get (program, GNOME_PARAM_POPT_CONTEXT, &context, NULL);
+                *return_ctx = context;
         }
 
         return 0;
