@@ -43,7 +43,7 @@
 #include <stdio.h>
 
 /* Must be before all other gnome includes!! */
-#include "gnome-i18nP.h"
+#include <glib/gi18n-lib.h>
 
 int main(int argc, char *argv[])
 {
